/*Listing 13-4.  The Circular Buffer (Queue) Package */

/*---------------------------------------------------------------*/
/*
 *  Filename:     Q P A C . C 
 *  Implements a circular queue in Microsoft C 4.0
 *
 *  Copyright  1986, 1987 Nabajyoti Barkakati
 *  Copies may be made for noncommerical, private purposes only.
 *
 */
/*---------------------------------------------------------------*/

#include <stdio.h>
#include <malloc.h>

#include "serial.h"

/*--------------------------------------------------------------*/
/* q _ s e t u p
 * Allocate a queue with room for specified number of characters.
 * The argument elemsize is the size of each data element.
 */

QTYPE *q_setup ( unsigned maxsize, unsigned elemsize )
{
    QTYPE *queue;
    
	if ((queue = (QTYPE *) calloc(1, sizeof(QTYPE))) == NULL)
	{
	   fprintf(stderr,"No room for data in queue setup!\n");
	   return(NULL);
    	}
    	queue->maxsize = maxsize;
    	queue->elemsize = elemsize;
	if ((queue->data = calloc(maxsize,elemsize)) == NULL) 
	{
    		fprintf(stderr,"Ran out of memory in queue setup!\n");
    	}
    	
	/* Initialize queue */

	queue->front = -1;
    	queue->rear = -1;
    	queue->count = 0;
    	return(queue);
}

/*-------------------------------------------------------------*/
/* q _ r e s e t
 * Deallocate a queue and its data area.
 */

void q_reset ( QTYPE *queue )
{
    free (queue->data);
    free ((char *) queue);
}

/*-----------------------------------------------------------*/
/* q _ g e t f r o m
 * Copy next data element in queue to specified location.
 * Also return the pointer to this element.
 */

char *q_getfrom ( QTYPE *queue, char  *data )
{
    register int i, j;
    char *current;
    current = NULL;
    if(queue->front == -1) return(current);
/* Else retrieve data.  Copy elemsize characters into data. */
    if (queue->elemsize == 1){
       current = &(queue->data[queue->front]);
       *data = *current;
    }
   else {
   for(i=0,j=queue->front * queue->elemsize;
           i<queue->elemsize; i++, j++){
       *(data+i) = queue->data[j];
    }
    current =
        &(queue->data[queue->front * queue->elemsize]);
    }
    queue->count--;
    if(queue->count == 0) {
/* The queue is empty.  Reset fron and rear, and the count. */
        queue->front = queue->rear = -1;
        return(current);
   }
/* Increment front index and check for wraparound */
    if(queue->front == queue->maxsize-1) {
        queue->front = 0;
    }
    else {
        queue->front++;
    }
        return(current);
    }

/*--------------------------------------------------------*/
/* q _ p u t o n
 * Put a data element into queue.
 */

int q_puton ( QTYPE *queue, char  *data )
{
register int i, j;

	/* First check if queue is full.  Return 0 if full. */

	if(queue->count == queue->maxsize) 
		return(0);

	/* Else, adjust rear and check for wraparound */

	if(queue->rear == queue->maxsize-1) 
	{
        	queue->rear = 0;
	}
	else 
	{
        	queue->rear++;
	}
	
	/* Save data in queue.  Copy elemsize characters */

	if (queue->elemsize == 1) 
	{
        	queue-> data[queue->rear] = *data;
	}
	else 
	{
        	for(i=0, j=queue->rear * queue->elemsize;
			i<queue->elemsize; i++, j++) 
		{
			queue->data[j] = *(data+i);
		}
	}
	queue->count++;
	if(queue->front == -1) 
		queue->front = 0;
   	return(1); 			/* Successfully inserted element */
}
/*-------------------------------END OF FILE:  QPAC.C----------------------*/

