/* Definitions for the 8259 Programmable Interrupt Controller */

#define	P8259_0		0x20
#define P8259_1		0x21
#define END_OF_INT 0x20     /* Nonspecific EOI              */

/* Address of BIOS data area (segment 40h, ofset 0) */

#define BIOS_DATA	(( int far * )( 0x400000L ))

/* Some interrupt vectors */

#define BIOS_RS232  0x14

/* The address of the comm port is in the integer
 * 'comport'.  This variable is initialized by reading from 
 * the BIOS data area at segment 0x40.
 */

#define IER                (comport + 1)  /* Interrupt Enable Register */
#define IIR                (comport + 2)  /* Interrupt Identification  */
#define LCR                (comport + 3)  /* Line Control Register     */
#define MCR                (comport + 4)  /* Modem Control Register    */
#define LSR                (comport + 5)  /* Line Status Register      */
#define MSR                (comport + 6)  /* Modem Status Register     */

/* Individual Interrupt Enable Numbers */

#define RDAINT  1
#define THREINT 2
#define RLSINT  4
#define MSINT   8


/* Function to get bit 0 of an integer */

#define bit0(i)      (i & 0x0001)

#define _COM1 	0
#define _COM2 	1
#define _COM3 	2
#define _COM4 	3

/* Modem Control Register value */

#define MCRALL 15         /* (DTR, RTS, OUT1 and OUT2 = 1)  */
#define MCROFF 0          /* Everything off                 */

/* Interrupt Enable Register value to turn on/off all int.  */

#define IERALL  (RDAINT+THREINT+RLSINT+MSINT)
#define IEROFF  0

/* Some masks for turning interrupts off */
#define THREOFF 0xfd

/* Interrupt identification numbers */

#define MDMSTATUS   0
#define TXREGEMPTY  2
#define RXDATAREADY 4
#define RLINESTATUS 6



typedef struct QTYPE {
    int count;
    int front;
    int rear;
    int elemsize;
    int maxsize;
    char *data;
} QTYPE;

typedef struct VTBLTYPE {
    char *vname;
    int value;
} VTBLTYPE;

typedef struct NTBLTYPE {
    char     *pname;
	VTBLTYPE *vtblentry;
    int      *p_value;
    VTBLTYPE *p_vtable;
    int      vtblsize;
} NTBLTYPE;


/* Function prototypes from SERIAL.C */

/* Use interrupt attribute to define C function as interrupt handler */

void interrupt cdecl far s_mainhndlr ( void );
int s_setup ( void );
static void s_intinit ( void ); 
void s_cleanup ( void );
static void s_intoff ( void );
void s_setcommparams ( void );
int s_sendchar( int ch );
int s_rcvchar ( void );
int s_txqempty ( void );

/* From QPAC.C */

QTYPE *q_setup ( unsigned maxsize, unsigned elemsize );
void q_reset ( QTYPE *queue );
char *q_getfrom ( QTYPE *queue, char  *data );
int q_puton ( QTYPE *queue, char  *data );

